/*jslint browser:true, devel:true, white:true, vars:true, eqeq:true */
/*global $:false, intel:false, Mustache:false*/
/*
 * Copyright (c) 2012, Intel Corporation. All rights reserved.
 * File revision: 15 October 2012
 * Please see http://software.intel.com/html5/license/samples 
 * and the included README.md file for license terms and conditions.
 */

var onDeviceReady=function(){                             // called when Cordova is ready
   if( window.Cordova && navigator.splashscreen ) {     // Cordova API detected
        navigator.splashscreen.hide() ;                 // hide splash screen
    }
} ;
document.addEventListener("deviceready", onDeviceReady, false) ;

var months = ['Janeiro','Fevereiro','Março','Abril','Maio','Junho','Julho','Agosto','Setembro','Novembro','Dezembro'];

/* manage data needed for app (teams from Futliga) */
var dataSource = {
    
    mustRefreshFollowed: true,
    
    /* parameters for Futliga pages */
    parmGender: '',
    parmModality: '',
    parmCategory: '',
    parmType: '',
    parmName: '',
    
    /* base url for all Futliga pages */
    baseUrl: 'https://www.futliga.com.br',
    
    /* url to list Futliga teams */
    listUrl: '/futliga/associado/equipes/associado-equipes-obter-lista-detalhada.asp',
    
    /* url to get Futliga team details */
    detailsUrl: '/futliga/associado/associado-detalhar-equipe.asp',
    
    /* url to get Futliga team schedule */
    scheduleUrl: '/futliga/associado/equipes/associado-equipes-obter-agenda.asp',

    /* url for team images */
    teamImageUrl: '/imagens/distintivos/',
    
    /* patterns for primary page */
    noResultsPattern: 'Nenhum resultado encontrado',
    teamIdPattern: ':DetalharEquipe(',
    pageNumPattern: '(\\d de \\d)',
    
    /* patterns for secondary page */
    addressNamePattern: 'id="hidMapaNome" value="<b>',
    addressTextPattern: 'id="hidMapaEnderecoCompleto" value="',
    addressLatitudePattern: 'id="hidLatitudeOrigem"   value="',
    addressLongitudePattern: 'id="hidLongitudeOrigem"  value="',
    homeSchedulePattern: '<div id="tabs-detalhe-agenda-mandante">',
    guestSchedulePattern: '<div id="tabs-detalhe-agenda-visitante">',
    
    /* total number of pages */
    totalPages: 1,
    
    searchResult: [],
    teams: [],
    followed: [],
    
    /* get data for given team, identified by its page & index on the page */
    getTeam: function(teamId) {
        
        if(typeof teamId === 'undefined' || teamId === ''){
            return {};
        }
        
        for(var i = 0; i < dataSource.followed.length; i++){
            if(dataSource.followed[i].id == teamId){
                return dataSource.followed[i];
            }
        }
        
        for(var j = 0; j < dataSource.teams.length; j++){
            if(dataSource.teams[j].id == teamId){
                return dataSource.teams[j];
            }
        }
        
        var team = {
            id: teamId,
            elementId: 'team-'+teamId,
            name: '',
            image: '',
            colors: '',
            addressName: '',
            addressText1: '',
            addressText2: '',
            addressText3: '',
            addressURI: '',
            schedule: []
        };
        
        team = dataSource.requestTeamDetails(team);
        
        dataSource.teams.push(team);
        
        return team;
    },
    
    /* initialize data source */
    init: function(gender, modality, category, type, teamName) {
        dataSource.parmGender = gender;
        dataSource.parmModality = modality;
        dataSource.parmCategory = category;
        dataSource.parmType = type;
        dataSource.parmName = teamName;
    },
    
    /* flush cached data */
    flush: function() {
        dataSource.searchResult = [];
        dataSource.totalPages = 1;
    },
    
    /* get data for all teams of given page */
    search: function(page) {
        var teams = [];
        if(page in dataSource.searchResult){
            teams = dataSource.searchResult[page];
        }
        else{
            teams = dataSource.requestTeamList(page);
            dataSource.searchResult[page] = teams;
        }
        
        return $.Deferred().resolve(teams);
    },
    
    getFollowedTeams: function() {
        if(dataSource.mustRefreshFollowed){
            dataSource.refreshFollowedTeams();
        }
        return $.Deferred().resolve(dataSource.followed);
    },
    
    refreshFollowedTeams: function() {
        var followedTeamsStr = localStorage.getItem('followedTeams');
        var followedTeams = [];
        dataSource.followed = [];
        
        if(followedTeamsStr !== null){
            followedTeams = JSON.parse(followedTeamsStr);
            for(var i = 0; i < followedTeams.length; i++){
                var team = dataSource.getTeam(followedTeams[i]);
                dataSource.followed.push(team);
            }
        }
        
        dataSource.mustRefreshFollowed = false;
    },
    
    isFollowed: function(teamId){
        var followedTeamsStr = localStorage.getItem('followedTeams');
        var followedTeams = [];
        
        if(followedTeamsStr !== null){
            followedTeams = JSON.parse(followedTeamsStr);
            for(var i = 0; i < followedTeams.length; i++){
                if(followedTeams[i] == teamId){
                    return true;
                }
            }
        }
        
        return false;
    },
    
    requestTeamList: function(page){
        var parameters = "";
        parameters += "sexo=" + dataSource.parmGender;
        parameters += "&modalidade=" + dataSource.parmModality;
        parameters += "&idade=" + dataSource.parmCategory ;
        parameters += "&mandantevisitante=" + dataSource.parmType;
        parameters += "&pagina=" + ((page == 1) ? '' : page.toString());
        parameters += "&pesquisa=" + dataSource.parmName;
        
        /* request the list of teams */
        var response = $.ajax({
            type: "GET",
            url: dataSource.baseUrl + dataSource.listUrl,
            data: parameters,
            async: false});

        var teams = [];
        var totalPages = 1;
        
        if(response.responseText.indexOf(dataSource.noResultsPattern) == -1){
            /* if any team is found make a screen scrapping to transform the resulting html into json */
            var fromPos = response.responseText.indexOf(dataSource.teamImageUrl);
            var toPos = 0;
        
            if(fromPos > -1){
                var pageNumPos = response.responseText.search(dataSource.pageNumPattern);
                totalPages = response.responseText.substring(
                    response.responseText.indexOf(' de ', pageNumPos) + 4,
                    response.responseText.indexOf('<', pageNumPos)).replace(/(\r\n|\n|\r)/gm,"").trim();
            }

            while(fromPos > -1){
                fromPos += dataSource.teamImageUrl.length;
                toPos = response.responseText.indexOf('"', fromPos);
                var image = dataSource.baseUrl + dataSource.teamImageUrl + response.responseText.substring(fromPos, toPos);

                fromPos = response.responseText.indexOf(dataSource.teamIdPattern, toPos) + dataSource.teamIdPattern.length;
                toPos = response.responseText.indexOf(')', fromPos);
                var id = response.responseText.substring(fromPos, toPos);

                fromPos = response.responseText.indexOf('>', toPos) + 1;
                toPos = response.responseText.indexOf('<', fromPos);
                var name = response.responseText.substring(fromPos, toPos)
                    .replace(/(\r\n|\n|\r)/gm,"")
                    .trim();

                teams.push({
                    id: id,
                    elementId: 'team-'+id,
                    name: name,
                    image: image,
                    colors: '',
                    addressName: '',
                    addressText: '',
                    addressLatitude: 0,
                    addressLongitude: 0,
                    schedule: []                
                });
                
                fromPos = response.responseText.indexOf(dataSource.teamImageUrl, toPos);
            }
        }
        else{
            teams[0] = {
                id: '0',
                elementId: 'team-0',
                name: dataSource.noResultsPattern,
                image: dataSource.baseUrl + dataSource.teamImageUrl + '0.jpg',
                colors: '',
                addressName: '',
                addressText1: '',
                addressText2: '',
                addressText3: '',
                addressLatitude: 0,
                addressLongitude: 0,
                schedule: []
            };
        }
        
        /* set the total number of pages */
        dataSource.totalPages = totalPages;
        
        return teams;
    },
    
    requestTeamDetails: function(team){
        
        var parameters = ("codigo=" + team.id);
        
        /* request team details */
        var response = $.ajax({
            type: "GET",
            url: dataSource.baseUrl + dataSource.detailsUrl,
            data: parameters,
            async: false});
        
        var fromPos = 0;
        var toPos = 0;
        var responsePart = {};
        
        if(team.name === ''){
            responsePart = dataSource.getResponseTextPart(
                response.responseText, '<meta property="og:title" content="', '"');
            team.name = responsePart.text;
            
            responsePart = dataSource.getResponseTextPart(
                response.responseText, '<meta property="og:image" content="', '"');
            team.image = responsePart.text;
            if(team.image == 'https://www.futliga.com.br/imagens/cabecalho/logomarca-futliga.gif'){
                team.image = dataSource.baseUrl + dataSource.teamImageUrl + '0.jpg';
            }
        }
        
        fromPos = response.responseText.indexOf('Cores:');
        for(var i = 0; i <= 6; i++){
            toPos = response.responseText.substring(fromPos).indexOf('<br />') + fromPos;
            
            if(i < 6){
                fromPos = toPos + 6;
            }
        }
        team.colors = response.responseText.substring(fromPos, toPos).trim();
        var text = response.responseText.substring(toPos);
        
        if(text.indexOf(dataSource.addressNamePattern) != -1)
        {
            /* Address name */
            responsePart = dataSource.getResponseTextPart(text, dataSource.addressNamePattern, '<');
            team.addressName = responsePart.text;

            /* Address text */
            text = text.substring(responsePart.lastPos);
            responsePart = dataSource.getResponseTextPart(text, dataSource.addressTextPattern, '<br/><br/>" />');
            team.addressText1 = responsePart.text.substr(0, responsePart.text.indexOf('<br/>'));
            responsePart.text = responsePart.text.substr(responsePart.text.indexOf('<br/>') + 5);
            team.addressText2 = responsePart.text.substr(0, responsePart.text.indexOf('<br/>'));
            responsePart.text = responsePart.text.substr(responsePart.text.indexOf('<br/>') + 5);
            team.addressText3 = responsePart.text;

            /* Address latitude */
            text = text.substring(responsePart.lastPos);
            responsePart = dataSource.getResponseTextPart(text, dataSource.addressLatitudePattern, '"');
            var latitude = responsePart.text;

            /* Address longitude */
            text = text.substring(responsePart.lastPos);
            responsePart = dataSource.getResponseTextPart(text, dataSource.addressLongitudePattern, '"');
            var longitude = responsePart.text;
            
            team.addressURI = 'geo:?q=' + encodeURIComponent(
                team.addressText1.replace(', S/N', '') + ' ' +
                team.addressText2.replace(' - ', ' ').replace('/', ', ') + ' ' +
                team.addressText3.substr(5)); // ignore "CEP: " prefix
            
            /*team.addressURI = 'geo:?q=' +
                encodeURIComponent(latitude.replace('-', '- ') +','+ longitude.replace('-', '- '));*/
        }
        
        team.schedule = dataSource.requestTeamSchedule(team);
        
        return team;
    },
    
    requestTeamSchedule: function (team){
        
        var schedule = [];
        var parameters = ("codigo=" + team.id);
        
        var response = $.ajax({
            type: "GET",
            url: dataSource.baseUrl + dataSource.scheduleUrl,
            data: parameters,
            async: false});
        
        schedule.push.apply(schedule, dataSource.getSchedule(response.responseText, dataSource.homeSchedulePattern, true));
        schedule.push.apply(schedule, dataSource.getSchedule(response.responseText, dataSource.guestSchedulePattern, false));
        
        schedule.sort(function compare(a,b) {
          if (a.gameDateTime < b.gameDateTime){
             return -1;
          }
          if (a.gameDateTime > b.gameDateTime){
            return 1;
          }
          return 0;
        });
        
        return schedule;
    },
    
    getSchedule: function (text, scheduleBeginPattern, isHome){
        
        var index = 0;
        var schedule = [];
        var gameDayPattern = '<span style="font-size: 30px;">';
        var weekPattern = 'Semana \\d';
        
        if(text.indexOf(scheduleBeginPattern) != -1)
        {
            text = text.substring(text.indexOf(scheduleBeginPattern));
            text = text.substring(text.search(weekPattern));
            var weekTextLength = 0;
            
            for(var weekCount = 0; weekCount < 3; weekCount++)
            {
                text = text.substring(weekTextLength);
                var weekFirstPos = text.search(weekPattern);
                var weekLastPos = text.substring(weekFirstPos + 7).search(weekPattern);
                var weekText = text.substring(weekFirstPos, weekLastPos);
                weekTextLength = weekText.length;
                
                var responsePart = dataSource.getResponseTextPart(text, 'Semana ', '<br />');
                var week = responsePart.text.substring(0, responsePart.text.indexOf('/'));
                var year = responsePart.text.substring(responsePart.text.indexOf('/') + 1);
                
                var firstDateOfWeek = dataSource.getFirstDateOfWeek(week, year);
                var month = firstDateOfWeek.getMonth();
                var monthName = months[month];
                
                while(weekText.indexOf(gameDayPattern) != -1)
                {
                    responsePart = dataSource.getResponseTextPart(weekText, gameDayPattern, '</span>');
                    var day = responsePart.text;
                    
                    weekText = weekText.substring(responsePart.lastPos);
                    responsePart = dataSource.getResponseTextPart(weekText, ';">', '</span>');
                    var dayName = responsePart.text;
                    
                    weekText = weekText.substring(responsePart.lastPos);
                    responsePart = dataSource.getResponseTextPart(weekText, dataSource.teamIdPattern, ')');
                    var opponentId = responsePart.text;
                    
                    weekText = weekText.substring(responsePart.lastPos);
                    responsePart = dataSource.getResponseTextPart(weekText, 'src="', '"');
                    var opponentImage = dataSource.baseUrl + responsePart.text;
                    
                    weekText = weekText.substring(responsePart.lastPos);
                    responsePart = dataSource.getResponseTextPart(weekText, ');">', '</a>');
                    var opponentName = responsePart.text;
                    
                    weekText = weekText.substring(responsePart.lastPos);
                    responsePart = dataSource.getResponseTextPart(weekText, 'left;">', '&');
                    var gameTime = responsePart.text;
                    
                    weekText = weekText.substring(responsePart.lastPos);
                    responsePart = dataSource.getResponseTextPart(weekText, ');">', '</a>');
                    var gamePlace = responsePart.text;
                    
                    var gameDate = year +'-'+ month.toString() +'-'+ day;
                    
                    var addressURI = '';
                    if(!isHome)
                    {
                        var opponent = dataSource.getTeam(opponentId);
                        addressURI = opponent.addressURI;
                    }
                
                    schedule[index] = {
                        opponentId: opponentId,
                        opponentName: opponentName,
                        opponentImage: opponentImage,
                        gameDateTime: gameDate +' '+ gameTime,
                        text1: dayName +', '+ day +' de '+ monthName,
                        text2: gameTime +' - '+ gamePlace,
                        isHome: isHome,
                        addressURI: addressURI
                    };
                    
                    index++;
                }
            }
        }
        
        return schedule;
    },
    
    getResponseTextPart: function(responseText, startPattern, endPattern){
        var fromPos = responseText.indexOf(startPattern) + startPattern.length;
        var toPos = responseText.indexOf(endPattern, fromPos);
        return {text: responseText.substring(fromPos, toPos).trim(), lastPos: toPos};
    },
        
    getFirstDateOfWeek: function (w, y) {
        var simple = new Date(y, 0, 1 + (w - 1) * 7);
        var dow = simple.getDay();
        var weekStartDate = simple;
        if (dow <= 4){
            weekStartDate.setDate(simple.getDate() - simple.getDay() + 1);
        }
        else{
            weekStartDate.setDate(simple.getDate() + 8 - simple.getDay());
        }
        return weekStartDate;
    }
};


/* manage creation of views (and one-time binding of data to each view) */
var viewController = {
    
    isHomePageActive: false,
    
    /* total number of pages */
    nPrimaryPagesTotal: 0,
    
    /* total number of page skeletons created so far */
    nPrimaryPagesInit: 0,
    
    /* total number of pages created (with content) so far */
    nPrimaryPagesCreated: 0,
    
    /* init input controls with cached info */
    preinit: function() {
        
        viewController.init();
        
        $(document).on("pageshow", "#homePage", function (event, ui) {
            viewController.isHomePageActive = true;
            dataSource.getFollowedTeams().done(function(data) {
                $('#homePage').find(':jqmData(role=content)').empty();
                if(data && data.length > 0){
                    $(Mustache.render(viewController.primaryPageContentTmpl, data))
                        .prependTo($('#homePage').find(':jqmData(role=content)'))
                        .listview();
                }
            });
        });
        
        $(document).on("pagehide", "#homePage", function (event, ui) {
            viewController.isHomePageActive = false;
        });
    },
    
    // initialize app upon start
    init: function() {
        // get templates for creating page skeletons & data-bound content 
        viewController.primaryPageSkeletonTmpl = $('#primaryPageSkeletonTmpl').html();
        viewController.primaryPageContentTmpl = $('#primaryPageContentTmpl').html();
        viewController.secondaryPageContentTmpl = $('#secondaryPageContentTmpl').html();
        viewController.teamAddressContentTmpl = $('#teamAddressContentTmpl').html();
        viewController.teamScheduleContentTmpl = $('#teamScheduleContentTmpl').html();
        
        // bind events
        $(document).on('click', '.perform-search', viewController.performSearch);
        $(document).on('pageinit', '.primaryPage', viewController.createPrimaryPage);
        $(document).on('click', '.teams > li', viewController.createSecondaryPage);
        $(document).on('click', '.follow-unfollow', viewController.followUnfollow);
        $(document).on('click', '.go-to-team', viewController.goToTeam);     
        $(document).on('click', '.exit', viewController.exit);     
    },
    
    performSearch: function(event) {
        event.preventDefault();
        
        if(viewController.nPrimaryPagesCreated > 0){
            viewController.reset();
        }

        var teamName = $('#input-name').val();
        var gender = $('#input-gender').children(":selected").attr("id");
        var modality = $('#input-modality').children(":selected").attr("id");
        var category = $('#input-category').children(":selected").attr("id");
        var type = ''; // Always list all types


        dataSource.init(gender, modality, category, type, teamName);

        dataSource.search(1).done(function() {
            viewController.initPrimaryPages();
        });
    },
    
    followUnfollow: function(){
        var followedTeams = [];
        var followedTeamsStr = localStorage.getItem('followedTeams');
        var teamId = $('#team').attr('team-id');
        
        if(followedTeamsStr !== null){
            followedTeams = JSON.parse(followedTeamsStr);
        }
        
        if($(this).attr('btn-action') == 'follow'){
            followedTeams.push(teamId);
        }
        else{
            for(var i = 0; i < followedTeams.length; i++){
                if(followedTeams[i] == teamId){
                    followedTeams.splice(i, 1);
                }
            }
        }
        
        localStorage.setItem('followedTeams', JSON.stringify(followedTeams));
        dataSource.mustRefreshFollowed = true;
        viewController.toggleFollowUnfollowBtn($(this));
    },
    
    toggleFollowUnfollowBtn: function(button){
        if(button.attr('btn-action') == 'follow'){
            button.find('.ui-button-text').text('Deixar de seguir');
            button.attr('btn-action', 'unfollow');
        }
        else{
            button.find('.ui-button-text').text('Seguir equipe');
            button.attr('btn-action', 'follow');
        }
    },
    
    goToTeam: function(){
        var teamId = $(this).context.id;
        var team = dataSource.getTeam(teamId);
        viewController.createSecondaryPage('', team, true);
    },
    
    exit: function(){
        navigator.app.exitApp();
    },
        
    /* reset all views and get newest data */
    reset: function() {
        
        /* clear all primary page content */
        $('.primaryPage').remove();
        
        /* flush cache data */
        dataSource.flush();
        
        /* reset viewController */
        viewController.nPrimaryPagesCreated = 0;
        viewController.nPrimaryPagesInit = 0;
        viewController.nPrimaryPagesTotal = 0;
    },
    
    
    /* create all primary page skeletons, insert into dom & load default page */
    initPrimaryPages: function() {
        /* set number of pages */
        viewController.nPrimaryPagesTotal = dataSource.totalPages;

        /* need to create additional page skeletons */
        if (viewController.nPrimaryPagesInit < viewController.nPrimaryPagesTotal) {
        
            /* create additional page skeletons */
            var $pages = $();
            for (var pagenum = viewController.nPrimaryPagesInit + 1; pagenum <= viewController.nPrimaryPagesTotal; pagenum++) {
                $pages = $pages.add(viewController.initPrimaryPage(pagenum));
            }
            $pages.insertAfter('#searchPage');

            /* update the number of page skeletons created */
            viewController.nPrimaryPagesInit = viewController.nPrimaryPagesTotal;

            /* tell jQuery Mobile to process new pages (will load topmost=1st primary page by default at start) */
            $.mobile.initializePage();
            
            $.mobile.changePage('#1');
        }
    },
    
    /* create and return skeleton of primary page */
    initPrimaryPage: function(pagenum) {
        
        /* create page skeleton html node */
        var $page = $(viewController.primaryPageSkeletonTmpl)
            .attr('id', pagenum)        
            .attr('data-url', '#' + pagenum);
        
        /* link to previous page (if any) */
        if (pagenum == 1) {
            $page.find('.prev').addClass('ui-disabled');
        } else {
            $page.find('.prev').attr('data-rel', 'back');
        }
        
        /* link to next page (if any) */
        if (pagenum == viewController.nPrimaryPagesTotal) {
            $page.find('.next').addClass('ui-disabled');
        } else {
            $page.find('.next').attr('href', '#' + (pagenum + 1));
        }
        
        /* hide sub-header until main content inserted */
        $page.find(':jqmData(role=sub-header)').hide();
        
        /* return newly created page */
        return $page;
        
    },
    
    /* create primary page (insert list of movies for given page) */
    createPrimaryPage: function() {
        
        /* get current page */
        var $page = $(this);
        var pageNum = parseInt($(this).attr('id'));
        
        if(pageNum <= viewController.nPrimaryPagesCreated){
            /* avoid creating the same page twice */
            return;
        }
        
        /* get team data */
        dataSource.search(pageNum).done(function(data) { 
            /* render listview template with data */
            $(Mustache.render(viewController.primaryPageContentTmpl, data))                
                .prependTo($page.find(':jqmData(role=content)'))
                .listview();
            
            /* bookkeeping to keep track of # pages created */
            viewController.nPrimaryPagesCreated += 1;
            
            /* page creation completed, can hide spinner & show nav sub-header */
            $page.find(':jqmData(role=sub-header)').show();
            $('#team-0').find('.team').attr('href', '');
            $('#team-0').find('.team').prop('onclick', null).off('click');
        });
    },
    
    /* update secondary page (override content with details for given movie) */
    createSecondaryPage: function(event, team, scrollToTop) {
        
        if(typeof team === 'undefined'){
            var teamId = $(this).attr('id').substring(5);
            team = dataSource.getTeam(teamId);
        }
        
        $('#secondaryPage').find(':jqmData(role=content)').empty();
        
        $('#secondaryPage').find(':jqmData(role=content)')
            .html($(Mustache.render(viewController.secondaryPageContentTmpl, team)));
        
        if(team.addressName !== ''){
            $('#team-address').html($(Mustache.render(viewController.teamAddressContentTmpl, team)));
        }
        
        if(team.schedule.length > 0){
            $('#team-schedule').html($(Mustache.render(viewController.teamScheduleContentTmpl, team.schedule)));
        }
        
        var action = $('#follow-unfollow').attr('btn-action');
        var isFollowed = dataSource.isFollowed(team.id);
        
        if((action == 'follow' && isFollowed) || (action == 'unfollow' && !isFollowed)){
            viewController.toggleFollowUnfollowBtn($('#follow-unfollow'));
        }
        
        $('#secondaryPage').enhanceWithin();
        
        if(scrollToTop){
            $.mobile.silentScroll(0);
        }
    }
    
};

/* override jQuery Mobile transitioning style defaults */
$(document).bind('mobileinit', function () {
    $.mobile.defaultPageTransition = 'none';
    $.mobile.defaultDialogTransition = 'none';
});

/* avoid going back to splash screen */
$(window).on("navigate", function (event, data) {
    if (data.state.direction == 'back' && data.state.hash == '#splashScreen') {
        event.preventDefault();
        $(window).off("navigate");
    }
});

$(document).on('pageshow','#splashScreen',function(){
    setTimeout(function(){
        viewController.preinit();
        $.mobile.changePage('#homePage');
    }, 1000);
});